import math, random, pygame, copy
pygame.init()
from pygame.locals import *


import myObjects, myFrames
from myConstants import *
import PW15_variables as customVariables


	
		
class You(myObjects.GameObject):  
	def __init__(self, images, size, centerPosition, riverBorders):
		myObjects.GameObject.__init__(self, images, size, centerPosition)
		self.ferryDirection=random.randrange(8)
		self.direction=self.ferryDirection
		self.setNextAnimation(customVariables.dirToText[self.ferryDirection],-1,True)
		self.speed=0.0
		self.deltaAngle=1
		self.deltaSpeedRow=1
		self.deltaSpeedRiver=0.01
		self.deltaSpeedDec=0.001
		self.deltaMov=1
		self.minSpeed=0.9
		self.collision=False
		self.riverLeft,self.riverTop=riverBorders[0]
		self.riverRight,self.riverBottom=riverBorders[1]
		self.gold=0
		self.inFlow=False
		self.inVortex=False
		self.riverDirection=0
		self.loading=False
		self.unloading=False
		self.lastRock=[-1,-1]

	def row_old_movements(self,left,right):
		if left and right:
			if not self.inFlow or abs(self.ferryDirection-self.riverDirection) in [0,1,2,7,6]:
				#print self.inFlow,self.ferryDirection,self.riverDirection,abs(self.ferryDirection-self.riverDirection)
				self.speed+=self.deltaSpeedRow
				self.direction=self.ferryDirection
		elif left:
			self.ferryDirection=(self.ferryDirection+self.deltaAngle)%8
			addSpeed=0
			self.setNextAnimation(customVariables.dirToText[self.ferryDirection],-1,True)
		elif right:
			self.ferryDirection=(self.ferryDirection-self.deltaAngle)%8
			addSpeed=0
			self.setNextAnimation(customVariables.dirToText[self.ferryDirection],-1,True)
		#print 'd',self.direction,'+',self.ferryDirection,'s',self.speed,'+',addSpeed
		#self.sumDirAndSpeed(self.ferryDirection,addSpeed)
		
	def row(self,row):
		if row==0:
			self.speed+=self.deltaSpeedRow
		self.ferryDirection=(self.ferryDirection+row*self.deltaAngle)%8
		self.direction=self.ferryDirection
		self.setNextAnimation(customVariables.dirToText[self.ferryDirection],-1,True)

		
	def updatePosition(self):
		#Sprint self.speed,self.direction,self.inFlow,self.riverDirection,'dx',round(self.deltaMov*self.speed*math.cos(self.direction*math.pi/4)),'dy',round(self.deltaMov*self.speed*math.sin(self.direction*math.pi/4))
		if self.inVortex:
			self.setNextAnimation(customVariables.dirToText[random.randrange(8)],-1,True)
			
		elif self.inRiver():
			self.rect.left+=round(self.deltaMov*self.speed*math.cos(self.direction*math.pi/4))
			self.rect.top-=round(self.deltaMov*self.speed*math.sin(self.direction*math.pi/4))	
			if self.inFlow:
				#if self.direction==self.riverDirection:
				if abs(self.direction-self.riverDirection) in [0,1,2,7,6]:
					self.speed+=self.deltaSpeedRiver
				else:
					self.speed=max(self.minSpeed,self.speed*0.75)
				self.direction=self.riverDirection
		else:
			self.bounce()
			#self.speed*=0.75
			#self.direction=(self.direction+4)%8
			#print 'crash',self.direction,self.speed
		

		#decrease speed
		if self.speed>self.minSpeed:
			self.speed-=self.deltaSpeedDec

			
	def bounce(self):
		#print 'bounce',self.speed,self.direction
		self.speed=self.minSpeed
		if self.direction in [7,0,1]: self.rect.left-=self.deltaMov
		if self.direction in [3,4,5]: self.rect.left+=self.deltaMov
		if self.direction in [1,2,3]: self.rect.top+=self.deltaMov
		if self.direction in [5,6,7]: self.rect.top-=self.deltaMov
		self.direction=(self.direction+4)%8
		#print 'after',self.speed,self.direction,'dx',round(self.deltaMov*self.speed*math.cos(self.direction*math.pi/4)),'dy',round(self.deltaMov*self.speed*math.sin(self.direction*math.pi/4))	
	
	def inRiver(self):
		leftCheck=self.rect.left>self.riverLeft or self.direction in [6,7,0,1,2]
		rightCheck=self.rect.right<self.riverRight or self.direction in [2,3,4,5,6]
		topCheck=self.rect.top>self.riverTop or self.direction in [4,5,6,7,0]
		bottomCheck=self.rect.bottom<self.riverBottom or self.direction in [0,1,2,3,4]
		return leftCheck and rightCheck and topCheck and bottomCheck



class Soul(myObjects.GameObjectReusable):
	def __init__(self, name, size, centerPosition, ferryBorders, hasCoin):
		self.name=name
		myObjects.GameObjectReusable.__init__(self, myFrames.getImageFrames(self.name), size, centerPosition)
		self.direction=random.choice([[0,1],[0,-1],[1,0],[-1,0]])
		self.speed=1
		self.deltaMov=1	
		self.hasCoin=hasCoin
		self.hit=False
		self.movCounter=FPS*3
		self.ferryLeft,self.ferryTop=ferryBorders[0]
		self.ferryRight,self.ferryBottom=ferryBorders[1]
		self.collisionRect=pygame.Rect([self.rect.left+self.size[0]/8,self.rect.top-self.size[1]/8],[self.size[0]/4,self.size[1]/4])
		if self.hasCoin:
			self.changeColorToAllFrames(pygame.Color(0,255,36),pygame.Color(225,225,0))
		else:
			self.changeColorToAllFrames(pygame.Color(0,255,36),pygame.Color(88,85,89))
		self.blocked=False
		
	def randomizeMovement(self):
		self.direction=random.choice([[0,1],[0,-1],[1,0],[-1,0]]+2*[self.direction])
		self.setNextAnimation(str(self.direction),-1,True)

	def inFerry(self):
		if not self.hit:
			if self.rect.left<=self.ferryLeft:
				self.direction=[1,0]
				self.setNextAnimation(str(self.direction),-1,True)
			elif self.rect.right>=self.ferryRight:
				self.direction=[-1,0]
				self.setNextAnimation(str(self.direction),-1,True)
			if self.rect.top<=self.ferryTop:
				self.direction=[0,-1]
				self.setNextAnimation(str(self.direction),-1,True)
			elif self.rect.bottom>=self.ferryBottom:
				self.direction=[0,1]
				self.setNextAnimation(str(self.direction),-1,True)
		return True
			
	def updatePosition(self):
		self.movCounter-=1
		
		if self.hit:
			self.deltaMov=10
			nextAnim=random.choice([[0,1],[0,-1],[1,0],[-1,0]])
			self.setNextAnimation(str(nextAnim),-1,True)
		else:
			if self.movCounter<=0:
				self.movCounter=FPS*3
				self.randomizeMovement()
		
		if self.inFerry() and not self.blocked:
			deltaX=int(self.deltaMov*self.speed*self.direction[0])
			deltaY=int(self.deltaMov*self.speed*self.direction[1])
			self.rect.left+=deltaX
			self.rect.top-=deltaY
			self.collisionRect.left+=deltaX
			self.collisionRect.top-=deltaY
			
class Bridge(myObjects.GameObject):
	def __init__(self, images, size, centerPosition):
		myObjects.GameObject.__init__(self, images, size, centerPosition)
		self.originalPosition=self.getPosition()
		self.lastVibrationX=1
		self.lastVibrationY=1
		self.changeColors=False

	def resetPosition(self):
		if self.getPosition()<>self.originalPosition:
			self.setPosition(self.originalPosition)
			#self.changeColorToAllFrames(pygame.Color(255,1,1),pygame.Color(246,159,73))
		
	def vibrate(self):
		self.lastVibrationX*=-1
		self.lastVibrationY*=-1
		self.rect.left+=random.choice([0,self.lastVibrationX])
		self.rect.top+=random.choice([0,self.lastVibrationY])
		#self.changeColorToAllFrames(pygame.Color(246,159,73),pygame.Color(255,1,1))

			
			
			

class MouseOar(myObjects.GameObject):
	def __init__(self, images, size, centerPosition):
		myObjects.GameObject.__init__(self, images, size, centerPosition)
		
	def updatePosition(self):
		self.rect.topleft=pygame.mouse.get_pos()
	
	def click(self, rect):
		collisionRect=pygame.Rect([self.rect.left+self.size[0]/8,self.rect.top-self.size[1]/8],[self.size[0]/4,self.size[1]/4])
		return collisionRect.colliderect(rect)

class RiverCell(myObjects.GameObject):
	def __init__(self, images, size, centerPosition):
		myObjects.GameObject.__init__(self, images, size, centerPosition)
		self.type='empty'

class RiverCellEmpty():
	def __init__(self):
		self.type='empty'		
				
class RiverCellFlow(RiverCell):
	def __init__(self, images, size, centerPosition, flow):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='flow'
		self.direction=flow
		self.setNextAnimation(flow,-1,True)

class RiverCellVortex(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='vortex'

class RiverCellGold(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)	
		self.type='gold'		
		
class RiverCellRock(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='rock'
		self.setNextAnimation(str(random.randrange(1,5)),-1,True)
		
class RiverCellStartDock(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='startdock'
		self.waitingSouls=True

class RiverCellFinishDock(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='finishdock'

class RiverCellDamned(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='damned'		

class RiverCellQuestion(RiverCell):
	def __init__(self, images, size, centerPosition):
		RiverCell.__init__(self, images, size, centerPosition)
		self.type='question'			

class LevelMiniature(myObjects.LevelMiniature):	
	def __init__(self, images, size, centerPosition, levelNumber, goldMax, goldBest, locked):
		self.goldMax=goldMax
		self.completed=(goldBest>0)
		font=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), 30)
		self.levelNumberPrint=font.render(str(levelNumber), True, pygame.Color('Black')).convert_alpha()
		font=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), 15)
		self.levelBestPrint=font.render(str(goldBest)+'/'+str(goldMax), True, pygame.Color('Black')).convert_alpha()
		
		myObjects.LevelMiniature.__init__(self, images, size, centerPosition, levelNumber, self.completed, locked)
		if self.locked: self.setNextAnimation('locked',-1,True)
		elif self.completed:
			if goldBest>=self.goldMax: self.setNextAnimation('completed_gold',-1,True)
			else: self.setNextAnimation('completed_silver',-1,True)
		
		
		

	def loadFrame(self):
		try:
			self.image=pygame.transform.scale(self.images[self.currentAnimationName][self.currentAnimationIndex], self.size)
			if self.lookingLeft:
				self.image=pygame.transform.flip(self.image, True, False)
			self.image=self.get_alpha_surface(self.image, 255, self.r, self.g, self.b)
			self.image.blit(self.levelNumberPrint,(self.size[0]/3,self.size[1]/4))
			if self.completed:
				self.image.blit(self.levelBestPrint,(2,self.size[1]-15))
				
		except:
			raise			

		
class Overprint(myObjects.GameObject):
	def __init__(self, screenSize, size, centerPosition, background=None):
		images={}
		if background==None:
			tmpSurf=pygame.Surface(size,pygame.SRCALPHA, 32).convert_alpha()
			color=pygame.Color('Grey')
			color.a=200
			tmpSurf.fill(color)
		else:
			tmpSurf=myDataloader.getImageFramesForAction(background)[0]
		images['default']=[tmpSurf]
		images['row,col']=[1,1]
		myObjects.GameObject.__init__(self, images, size, centerPosition)	


class ResultScreen(Overprint):
	def __init__(self, screenSize, currentLevel, goldNeed, goldBest, goldLast, soulLast, background=None):
		self.currentLevel=currentLevel
		self.size=screenSize#[0]*8/10,screenSize[1]*8/10
		self.centerPosition=screenSize[0]/2,screenSize[1]/2
		Overprint.__init__(self, screenSize, self.size, self.centerPosition, background)
		self.nextLevel=0
		self.resultImage=pygame.Surface(self.size, pygame.SRCALPHA, 32).convert_alpha()
		
		self.goldNeed=goldNeed
		self.goldBest=goldBest
		self.goldLast=goldLast
		self.soulLast=soulLast
		self.souls=[]		
		
		font=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), 15)
		texts=[]
		#texts.append('Level '+str(self.currentLevel))

		startRow=self.size[1]/12
		startCol=self.size[1]/8		
		row=0
		col=0
		
		self.goldTotal=0
		for soulName in self.soulLast.keys():
			for hasCoin in self.soulLast[soulName].keys():
				newSoulImage=myFrames.getImageFramesForAction(soulName)[0].copy()
				newSoulImage = pygame.PixelArray(newSoulImage)
				tolerance=0.1
				if hasCoin:
					newSoulImage.replace(pygame.Color(0,255,36), pygame.Color(225,225,0), tolerance)
				else:
					newSoulImage.replace(pygame.Color(0,255,36), pygame.Color(88,85,89), tolerance)
				self.resultImage.blit(newSoulImage.make_surface(),(startCol*(1+col),startRow*(2+row)))
				
				num=self.soulLast[soulName][hasCoin]
				value=hasCoin*(num*customVariables.SOUL_BONUS)+(not hasCoin)*(num*customVariables.SOUL_MALUS)
				self.goldTotal+=value
				texts.append(' x '+str(num)+' = '+str(value))
				#row+=startRow
				if row==7: col+=4
				row=(row+1)%8
		
		goldImage=myFrames.getImageFramesForAction('obj_river_gold.png')[0]
		self.resultImage.blit(goldImage,(startCol*(1+col),startRow*(2+row)))
		value=self.goldLast*customVariables.GOLD_BONUS
		texts.append(' x '+str(self.goldLast)+' = '+str(value))
		self.goldTotal+=value
		row+=startRow
		texts.append('TOT = '+str(self.goldTotal))
		if self.goldTotal>=self.goldNeed and self.goldTotal>self.goldBest: texts.append('!! NEW RECORD !!')
		texts.append('')
		texts.append('Press ESC to select another level')
		texts.append('Press RETURN to continue')
		
		row=0
		col=0
		for t in range(len(texts)):
			#print row,col,(startCol+startCol*(1+col),startRow*(2+row)),texts[t]
			line=font.render(texts[t], True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
			self.image.blit(line, (startCol+startCol*(1+col),startRow*(2+row)))
			if row==7: col+=4
			row=(row+1)%8
		
		self.update()
	
	def update(self):
		self.image.blit(self.resultImage,(0,0))
	
	def checkWinCondition(self):
		#return gold_If_Best, nextLevel
		if self.goldTotal>=self.goldNeed:
			return max(self.goldBest,self.goldTotal),self.currentLevel+1
		else:
			return 0, self.currentLevel
	


	
	
class PauseScreen(Overprint):
	def __init__(self, screenSize, currentLevel, background=None):
		self.currentLevel=currentLevel
		self.size=screenSize#[0]*8/10,screenSize[1]*8/10
		centerPosition=screenSize[0]/2,screenSize[1]/2
		Overprint.__init__(self, screenSize, self.size, centerPosition, background)
		self.update()
		
	def update(self):
		font=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), 15)
		texts=[]
		#texts.append('Level '+str(self.currentLevel))
		'''
		texts.append('Controls (change setting editing PW15_variables.py):')
		texts.append('Left Oar: '+ ', '.join([pygame.key.name(k).upper() for k in customVariables.left_oar]))
		texts.append('Right Oar: '+ ', '.join([pygame.key.name(k).upper() for k in customVariables.right_oar]))
		texts.append('Both Oars: '+ ', '.join([pygame.key.name(k).upper() for k in customVariables.both_oars]))
		texts.append('Level Restart: '+ ', '.join([pygame.key.name(k).upper() for k in customVariables.level_restart]))
		texts.append('Pause/Select Level: ESC')
		texts.append('')
		'''
		texts.append('Press ESC to select another level')
		texts.append('Press any key to continue')

		for t in range(len(texts)):
			line=font.render(texts[t], True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
			self.image.blit(line, (self.size[0]/25,self.size[1]/3+(t+1+7)*self.size[1]/15))

	
class LoseScreen(Overprint):
	def __init__(self, screenSize, currentLevel, background=None):
		self.currentLevel=currentLevel
		self.size=screenSize#[0]*8/10,screenSize[1]*8/10
		centerPosition=screenSize[0]/2,screenSize[1]/2
		Overprint.__init__(self, screenSize, self.size, centerPosition, background)
		self.update()
		
	def update(self):
		font=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), 15)
		texts=[]
		texts.append('You lose')
		texts.append('')
		texts.append('Press ESC to select another level')
		texts.append('Press any key to try again')
		

		for t in range(len(texts)):
			line=font.render(texts[t], True, pygame.Color('White'),pygame.Color('Black')).convert_alpha()
			self.image.blit(line, (self.size[0]/25,self.size[1]/4+(t+1)*self.size[1]/15))

		
		


			

