'''Game main module.

'''
import sys, pygame, random, math, os, time, traceback
import myDataloader, myResolution, myFrames, myLevels, myObjects
import PW15_levels as customLevels
import PW15_objects as customObjects
from myConstants import *
os.environ['SDL_VIDEO_CENTERED'] = '1'

	
from pygame.locals import *
pygame.init()
pygame.font.init()
pygame.mouse.set_visible(False)


clock = pygame.time.Clock()
debugOn=False
musicOn=True

screen=myResolution.getResolution(DEFAULT_SCREEN_SIZE)
pygame.display.set_caption('Charon\'s Obol - Forked Tail Team - PyWeek#15 ')
pygame.display.set_icon(myDataloader.load_image('icon.png'))
myFrames.loadAllFramesDetails()

try:
	myDataloader.load_music('555G.mp3')
except pygame.error: 
	print '=============================='
	print 'WARNING: music file not found.'
	print '=============================='
	
	
########################################
def createLevel(nextLevel,animations,inventory):
	screenSize=(screen.get_width(),screen.get_height())
	sortedLayers=[customObjects.Bridge,customObjects.RiverCell,customObjects.Soul,customObjects.You,customObjects.LevelMiniature,customObjects.Overprint,myObjects.GameText,customObjects.MouseOar]
	#print nextLevel,inventory
	#splashscreen and credits
	if nextLevel==SPLASHSCREEN:		
		currentLevel=myLevels.Splash(nextLevel,screenSize,'screen_forkedtaillogo.png',[pygame.Color('Red')], debugOn, musicOn, inventory)
	else:
		#start music
		if not pygame.mixer.music.get_busy() and musicOn:
				pygame.mixer.music.play(-1)		  
		#choose level
		if nextLevel==GAMEOVER:
			storyLength=1
			currentLevel=myLevels.Intro(nextLevel,screenSize,storyLength,'gameover', debugOn, musicOn, inventory)			
		elif nextLevel==GAMEINTRO:
			storyLength=3
			currentLevel=myLevels.Intro(nextLevel,screenSize,storyLength,'intro', debugOn, musicOn, inventory)
		elif nextLevel==LEVELSELECT:
			currentLevel=customLevels.PW15LevelSelection(nextLevel,screenSize,debugOn, musicOn, inventory)
		elif nextLevel>=LEVEL1 and nextLevel<YOUWIN:
			currentLevel=customLevels.LevelOne(nextLevel,screenSize,debugOn, musicOn, inventory, sortedLayers)				
		elif nextLevel==YOUWIN:
			storyLength=3
			currentLevel=myLevels.Intro(nextLevel,screenSize,storyLength,'youwin', debugOn, musicOn, inventory)		
		elif nextLevel==CREDITS:
			storyLength=1
			currentLevel=myLevels.Intro(nextLevel,screenSize,storyLength,'credits', debugOn, musicOn, inventory)
		else:
			#includes TITLESCREEN:
			menuText=['play','audio '+(not musicOn)*'on'+(musicOn)*'off','credits','quit']
			textColors=[pygame.Color(120,200,240,255),pygame.Color(214,205,154,255)]
			textPos=(screenSize[0]/8,screenSize[1]/2)
			backgroundImageName='screen_title1.png'

			currentLevel=customLevels.Title(nextLevel, screenSize, debugOn, musicOn, inventory, menuText, textColors, textPos, backgroundImageName)		
			#sys.exit('Level '+str(nextLevel)+' does not exist')	
			#sys.exit('END OF GAME')	
	return currentLevel	


def mainLoop(currentLevel):  
	screen.fill([0,0,0])
	screen.blit(currentLevel.backgroundImage, (0,0))	
	#level loop
	while currentLevel.levelCompleted==False:
		try:
			clock.tick(FPS) 
			#manage key events
			for event in pygame.event.get():
				currentLevel.eventsManagement(event)
							
			#update objects
			currentLevel.update()

			#draw background
			'''#scrolling screen
			if currentLevel.scrolled:
				screen.blit(currentLevel.backgroundImage, (0,0))
				currentLevel.scrolled=False
			'''
			if currentLevel.__class__==myLevels.Intro:
				screen.blit(currentLevel.backgroundImage, (0,0))
			currentLevel.levelObjects.clear(screen, currentLevel.backgroundImage)
			
			#draw objects
			'''#scrolling screen
			pygame.display.update(currentLevel.levelObjects.draw(screen, currentLevel.scrollDelta))
			'''
			pygame.display.update(currentLevel.levelObjects.draw(screen))
			
			#draw FPS
			FPSfont=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), 20)
			#screen.blit(FPSfont.render('FPS '+str(round(clock.get_fps(),1)), True, pygame.Color('Black'), pygame.Color('White')).convert_alpha(), (0,0))
			'''
			if currentLevel.currentLevel >= LEVEL1:
				text='FPS '+str(round(clock.get_fps(),1))#+' - Esc to Pause/Help'
				#text='FPS '+str(round(clock.get_fps(),1))+' - s'+str(currentLevel.you.speed)+' d'+str(currentLevel.you.direction)+' x'+str(int(currentLevel.you.deltaMov*currentLevel.you.speed*math.cos(currentLevel.you.direction*math.pi/4)))+' y'+str(int(currentLevel.you.deltaMov*currentLevel.you.speed*math.sin(currentLevel.you.direction*math.pi/4)))+' souls='+str(currentLevel.waitingSouls)	, True, pygame.Color('Black'), pygame.Color('White')).convert_alpha(), (0,screen.get_height()-20))	
				screen.blit(FPSfont.render(text, True, pygame.Color('Black'), pygame.Color('White')).convert_alpha(), (0,screen.get_height()-20))	
			'''	
			
			#refresh screen
			pygame.display.flip()
		except:
			raise
			FATALERROR=True
			errorfile=open('errors.log', 'a')
			errorfile.write('-----FATAL ERRROR-----'+'\n'+str(time.localtime())+'\n')
			traceback.print_exc(file=errorfile)
			print '-------------------------------------\nFATAL ERRROR. Please check errors.log\n-------------------------------------'
			break
	return currentLevel


def loadGameFromFile():
	try:
		inventory= myDataloader.loadgame('game001.sav')
		print 'Number of levels:',len(inventory['level'])
		return inventory
	except IOError:
		print 'Error in loading data from file!'
		return resetGame()
		
def saveGameOnFile(inventory):
	try:
		myDataloader.savegame('game001.sav',inventory)
		return True
	except IOError:
		print 'Error in saving data on file!'
	
def resetGame():
	inventory={}
	inventory['totalGold']=0
	inventory['totalSoul']={}	
	inventory['level']={}
	numOfLevels=len(os.listdir(myDataloader.levels_dir))
	print 'Number of levels:',numOfLevels
	for lev in range(1,numOfLevels+1):
		inventory['level'][lev]={}
		inventory['level'][lev]['best']=0
		inventory['level'][lev]['max']=0
		inventory['level'][lev]['need']=0
		inventory['level'][lev]['gold']=0
		inventory['level'][lev]['soul']={}
		for name in customVariables.LIST_OF_SOULS[:min(len(customVariables.LIST_OF_SOULS),(lev-1)/5+1)]:
			inventory['level'][lev]['soul'][name]={}
			inventory['level'][lev]['soul'][name][True]=0
			inventory['level'][lev]['soul'][name][False]=0
		
	return inventory

	
	
def main():
	global debugOn
	global musicOn	
	currentLevel=None
	nextLevel=SPLASHSCREEN
	FATALERROR=False
		
	#animations dictionary stores images to be used in every level
	animations={}

	#inventory stores user data across different levels (e.g. lives, point, etc...)
	#rem15/09/2012 inventory=resetGame()
	inventory=loadGameFromFile()
	
	while not FATALERROR:
		#create level
		currentLevel=createLevel(nextLevel,animations,inventory)
		
		#main loop
		try:
			currentLevel=mainLoop(currentLevel)
		except:
			print traceback.format_exc()
			traceback.print_exc(file=open("error.log","a"))
			sys.exit(1)
		
		#load animation with splashscreen
		'''#10/09/2012 no need to use animation variable: use myFrames.allFramesLoaded
		if currentLevel.currentLevel==SPLASHSCREEN:
			animations=currentLevel.animations
		'''
		#save current state for next level
		try:
			#print 'was',currentLevel.inventory
			inventory=currentLevel.inventory
			#print inventory
			debugOn=currentLevel.debugOn
			saveGameOnFile(inventory)
		except:
			print 'ERROR - resetting game'
			inventory=resetGame()
		
		#change level
		nextLevel=currentLevel.nextLevel
